#
# Simple AMC13 initialization and control program
#
"""
Eric Hazen
2012-03-14
"""

import readline
import sys
import amc13

def help():
  print "---General Commands---"
  print "I                       initialize AMC13 for TTC operation"
  print "Z                       zero all counters"
  print "S                       display status"
  print "---Expert Commands---"
  print "R <addr>                read address (one word)"
  print "R <addr> <count>        read address <count> words"
  print "W <addr> <data>         write one word"
  print "T1                      Switch to T1 for register access"
  print "T2                      Switch to T2 for register access"
  print "Q                       quit"

print "AMC13 Control Program rev 2012-03-14"

sn = amc13.sn1()
t1, t2 = amc13.connect(sn)
ver1 = t1.read("T1_FIRM_VER")     # read firmware versions
ver2 = t2.read("T2_FIRM_VER")
print "Found AMC13 S/N", sn, "T1 rev:", ver1, "T2 rev:", ver2
t1_ip, t2_ip = amc13.ip_addr(sn)
print "Using IP addresses T1=", t1_ip, "T2=", t2_ip

x = t1

help()

while 1:
  s = raw_input('>> ')
  a = s.split()
  if len(a) >= 1:
    cmd = a[0].upper()

    if cmd == 'I':
      t1.write("CONTROL0", 1)           # issue general reset to Virtex
      t2.write("CONTROL0", 1)           # issue general reset to Spartan
      t2.write("TTC_ENABLE", 0xfff)     # enable all TTC outputs
      print "AMC13 initialized, all TTC outputs enabled"

    elif cmd == 'S':
      print "AMC13 status (T1, T2 should normally agree)"
      print "  EvN T1: 0x%08x" % t1.read("TTC_EVN"), "T2: 0x%08x" % t2.read("TTC_EVN")
      print "  BcN T1: 0x%08x" % t1.read("TTC_BCN"), "T2: 0x%08x" % t2.read("TTC_BCN")
      print "  OrN T1: 0x%08x" % t1.read("TTC_ORN"), "T2: 0x%08x" % t2.read("TTC_ORN")
      # check for errors
      print "Errors (if any) below"
      ttcerr = False
      t1bce = t1.read("TTC_BCERROR")
      t2bce = t2.read("TTC_BCERROR")
      t1sbe = t1.read("TTC_SBERROR")
      t2sbe = t2.read("TTC_SBERROR")
      t1mbe = t1.read("TTC_MBERROR")
      t2mbe = t1.read("TTC_MBERROR")
      if t1bce or t2bce:
        ttcerr = True
        print "  NOTE:  orbit length errors seen (wrong BC0 spacing)"
        print "  BcE T1: 0x%08x" % t1bce, "T2: 0x%08x" % t2bce
      if t1sbe or t2sbe or t1mbe or t2mbe:
        ttcerr = True
        print "  NOTE:  TTC receiver bit errors detected.  Check fiber/connector"
        print "  SbE T1: 0x%08x" % t1sbe, "T2: 0x%08x" % t2sbe
        print "  Mbe T1: 0x%08x" % t1mbe, "T2: 0x%08x" % t2mbe        
      if not ttcerr:
        print "No errors detected"

    elif cmd == 'Z':
      print "Reset counters (T1 and T2 may not be exactly synchronized)"
      t2.write("CONTROL0",1)
      t1.write("CONTROL0",2)

    elif cmd == 'R':
      # see if first argument is a register name or a value (hex/decimal)
      if a[1].isdigit() or a[1].startswith("0x"):
        base = "ID"
        addr = int(a[1], 0)
      else:
        base = a[1]
        addr = 0
      v = x.read(base,addr)
      if len(a) == 3:
        n = int(a[2], 0)
        for i in range(n):
          v = x.read(base,addr+i)
          print "0x%04x: " % (addr+i), " 0x%08x" % v
      else:
        print "0x%04x: " % addr, " 0x%08x" % v
           
    elif cmd == 'W':
      o = int( a[1], 0)
      d = int( a[2], 0)
      x.write( "ID", d, o)

    elif cmd == "T1":
      x = t1
      print "Now using the Virtex Chip (T1)"

    elif cmd == "T2":
      x = t2
      print "Now using the Spartan Chip (T2)"

    elif cmd == "Q":
      sys.exit(0)

    else:
      help()
    
